@@ -7,7 +7,8 @@
 #include "term.h"
 #include "charset.h"
 
-#include "winpriv.h"  /* win_prefix_title */
+#include "winpriv.h"  /* win_prefix_title, win_update_now */
+#include "appinfo.h"  /* APPNAME, VERSION */
 
 #include <pwd.h>
 #include <fcntl.h>
@@ -39,8 +40,6 @@ int forkpty(int *, char *, struct termios *, struct winsize *);
 // http://www.tldp.org/LDP/abs/html/exitcodes.html
 #define mexit 126
 
-bool clone_size_token = true;
-
 string child_dir = null;
 
 static pid_t pid;
@@ -54,6 +53,16 @@ bool logging = false;
 #include <langinfo.h>
 #endif
 
+
+#define dont_debug_dir
+
+#ifdef debug_dir
+#define trace_dir(d)	show_info(d)
+#else
+#define trace_dir(d)	
+#endif
+
+
 static void
 childerror(char * action, bool from_fork, int errno_code, int code)
 {
@@ -209,7 +218,7 @@ child_update_charset(void)
 void
 child_create(char *argv[], struct winsize *winp)
 {
-  string lang = cs_lang();
+  trace_dir(asform("child_create: %s", getcwd(malloc(MAX_PATH), MAX_PATH)));
 
   // xterm and urxvt ignore SIGHUP, so let's do the same.
   signal(SIGHUP, SIG_IGN);
@@ -272,16 +281,28 @@ child_create(char *argv[], struct winsize *winp)
     signal(SIGTTOU, SIG_IGN);
 
     setenv("TERM", cfg.term, true);
+    // unreliable info about terminal application (#881)
+    setenv("TERM_PROGRAM", APPNAME, true);
+    setenv("TERM_PROGRAM_VERSION", VERSION, true);
 
-    if (lang) {
-      unsetenv("LC_ALL");
-      unsetenv("LC_COLLATE");
-      unsetenv("LC_CTYPE");
-      unsetenv("LC_MONETARY");
-      unsetenv("LC_NUMERIC");
-      unsetenv("LC_TIME");
-      unsetenv("LC_MESSAGES");
-      setenv("LANG", lang, true);
+    // If option Locale is used, set locale variables?
+    // https://github.com/mintty/mintty/issues/116#issuecomment-108888265
+    // Variables are now set in update_locale() which sets one of 
+    // LC_ALL or LC_CTYPE depending on previous setting of 
+    // LC_ALL or LC_CTYPE or LANG, stripping @cjk modifiers for WSL.
+    if (cfg.old_locale) {
+      //string lang = cs_lang();
+      string lang = cs_lang() ? cs_get_locale() : 0;
+      if (lang) {
+        unsetenv("LC_ALL");
+        unsetenv("LC_COLLATE");
+        unsetenv("LC_CTYPE");
+        unsetenv("LC_MONETARY");
+        unsetenv("LC_NUMERIC");
+        unsetenv("LC_TIME");
+        unsetenv("LC_MESSAGES");
+        setenv("LANG", lang, true);
+      }
     }
 
     // Terminal line settings
@@ -319,6 +340,11 @@ child_create(char *argv[], struct winsize *winp)
     exit(mexit);
   }
   else { // Parent process.
+    if (report_child_pid) {
+      printf("%d\n", pid);
+      fflush(stdout);
+    }
+
 #ifdef __midipix__
     // This corrupts CR in cygwin
     struct termios attr;
@@ -345,7 +371,9 @@ child_create(char *argv[], struct winsize *winp)
           dev += 3;
         else if (!strncmp(dev, "pts/", 4))
           dev += 4;
-        strncpy(ut.ut_id, dev, sizeof ut.ut_id);
+        //strncpy(ut.ut_id, dev, sizeof ut.ut_id);
+        for (uint i = 0; i < sizeof ut.ut_id && *dev; i++)
+          ut.ut_id[i] = *dev++;
 
         ut.ut_type = USER_PROCESS;
         ut.ut_pid = pid;
@@ -461,15 +489,64 @@ child_proc(void)
         // this avoids most partial updates, results in less flickering/tearing.
         static char buf[4096];
         uint len = 0;
+#if CYGWIN_VERSION_API_MINOR >= 74
+        if (cfg.baud > 0) {
+          uint cps = cfg.baud / 10; // 1 start bit, 8 data bits, 1 stop bit
+          uint nspc = 2000000000 / cps;
+
+          static ulong prevtime = 0;
+          static ulong exceeded = 0;
+          static ulong granularity = 0;
+          struct timespec tim;
+          if (!granularity) {
+            clock_getres(CLOCK_MONOTONIC, &tim); // cygwin granularity: 539ns
+            granularity = tim.tv_nsec;
+          }
+          clock_gettime(CLOCK_MONOTONIC, &tim);
+          ulong now = tim.tv_sec * (long)1000000000 + tim.tv_nsec;
+          //printf("baud %d ns/char %d prev %ld now %ld delta\n", cfg.baud, nspc, prevtime, now);
+          if (now < prevtime + nspc) {
+            ulong delay = prevtime ? prevtime + nspc - now : 0;
+            if (delay < exceeded)
+              exceeded -= delay;
+            else {
+              tim.tv_sec = delay / 1000000000;
+              tim.tv_nsec = delay % 1000000000;
+              clock_nanosleep(CLOCK_MONOTONIC, 0, &tim, 0);
+              clock_gettime(CLOCK_MONOTONIC, &tim);
+              ulong then = tim.tv_sec * (long)1000000000 + tim.tv_nsec;
+              //printf("nsleep %ld -> %ld\n", delay, then - now);
+              if (then - now > delay)
+                exceeded = then - now - delay;
+              now = then;
+            }
+          }
+          prevtime = now;
+
+          int ret = read(pty_fd, buf, 1);
+          if (ret > 0)
+            len = ret;
+        }
+        else
+#endif
         do {
           int ret = read(pty_fd, buf + len, sizeof buf - len);
+          //if (kb_trace) printf("[%lu] read %d\n", mtime(), ret);
           if (ret > 0)
             len += ret;
           else
             break;
         } while (len < sizeof buf);
+
         if (len > 0) {
           term_write(buf, len);
+          // accelerate keyboard echo if (unechoed) keyboard input is pending
+          if (kb_input) {
+            kb_input = false;
+            if (cfg.display_speedup)
+              // undocumented safeguard in case something goes wrong here
+              win_update_now();
+          }
           if (log_fd >= 0 && logging)
             write(log_fd, buf, len);
         }
@@ -592,20 +669,22 @@ grandchild_process_list(void)
     int thispid = atoi(pn);
     if (thispid && thispid != pid) {
       char * ctty = procres(thispid, "ctty");
-      if (0 == strcmp(ctty, tty)) {
-        int ppid = procresi(thispid, "ppid");
-        int winpid = procresi(thispid, "winpid");
-        // not including the direct child (pid)
-        ttyprocs = renewn(ttyprocs, nttyprocs + 1);
-        ttyprocs[nttyprocs].pid = thispid;
-        ttyprocs[nttyprocs].ppid = ppid;
-        ttyprocs[nttyprocs].winpid = winpid;
-        char * cmd = procres(thispid, "cmdline");
-        ttyprocs[nttyprocs].cmdline = cmd;
+      if (ctty) {
+        if (0 == strcmp(ctty, tty)) {
+          int ppid = procresi(thispid, "ppid");
+          int winpid = procresi(thispid, "winpid");
+          // not including the direct child (pid)
+          ttyprocs = renewn(ttyprocs, nttyprocs + 1);
+          ttyprocs[nttyprocs].pid = thispid;
+          ttyprocs[nttyprocs].ppid = ppid;
+          ttyprocs[nttyprocs].winpid = winpid;
+          char * cmd = procres(thispid, "cmdline");
+          ttyprocs[nttyprocs].cmdline = cmd;
 
-        nttyprocs++;
+          nttyprocs++;
+        }
+        free(ctty);
       }
-      free(ctty);
     }
   }
   closedir(d);
@@ -626,7 +705,7 @@ grandchild_process_list(void)
           procw[i] = 0x2007;  // FIGURE SPACE
     int wid = min(wcslen(procw), 40);
     for (int i = 13; i < wid; i++)
-      if ((cfg.charwidth ? xcwidth(procw[i]) : wcwidth(procw[i])) == 2)
+      if (((cfg.charwidth % 10) ? xcwidth(procw[i]) : wcwidth(procw[i])) == 2)
         wid--;
     procw[wid] = 0;
 
@@ -827,7 +906,7 @@ user_command(wstring commands, int n)
         // check for multi-line separation
         if (*cmdp == '\\' && cmdp[1] == '\n') {
           cmdp += 2;
-          while (isspace(*cmdp))
+          while (iswspace(*cmdp))
             cmdp++;
         }
       }
@@ -939,12 +1018,12 @@ setenvi(char * env, int val)
 static void
 setup_sync()
 {
-  if (cfg.geom_sync) {
+  if (sync_level()) {
     if (win_is_fullscreen) {
       setenvi("MINTTY_DX", 0);
       setenvi("MINTTY_DY", 0);
     }
-    else {
+    else if (!IsZoomed(wnd)) {
       RECT r;
       GetWindowRect(wnd, &r);
       setenvi("MINTTY_X", r.left);
@@ -952,6 +1031,8 @@ setup_sync()
       setenvi("MINTTY_DX", r.right - r.left);
       setenvi("MINTTY_DY", r.bottom - r.top);
     }
+    if (cfg.tabbar)
+      setenvi("MINTTY_TABBAR", cfg.tabbar);
   }
 }
 
@@ -959,27 +1040,30 @@ setup_sync()
   Called from Alt+F2 (or session launcher via child_launch).
  */
 static void
-do_child_fork(int argc, char *argv[], int moni, bool launch)
+do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size)
 {
+  trace_dir(asform("do_child_fork: %s", getcwd(malloc(MAX_PATH), MAX_PATH)));
   setup_sync();
 
+#ifdef control_AltF2_size_via_token
   void reset_fork_mode()
   {
     clone_size_token = true;
   }
+#endif
 
   pid_t clone = fork();
 
   if (cfg.daemonize) {
     if (clone < 0) {
       childerror(_("Error: Could not fork child daemon"), true, errno, 0);
-      reset_fork_mode();
+      //reset_fork_mode();
       return;  // assume next fork will fail too
     }
     if (clone > 0) {  // parent waits for intermediate child
       int status;
       waitpid(clone, &status, 0);
-      reset_fork_mode();
+      //reset_fork_mode();
       return;
     }
 
@@ -1015,6 +1099,7 @@ do_child_fork(int argc, char *argv[], int moni, bool launch)
       }
 
       chdir(set_dir);
+      trace_dir(asform("child: %s", set_dir));
       setenv("PWD", set_dir, true);  // avoid softlink resolution
       // prevent shell startup from setting current directory to $HOME
       // unless cloned/Alt+F2 (!launch)
@@ -1062,9 +1147,14 @@ do_child_fork(int argc, char *argv[], int moni, bool launch)
 #endif
 
     // provide environment to clone size
-    if (clone_size_token) {
-      setenvi("MINTTY_ROWS", term.rows);
-      setenvi("MINTTY_COLS", term.cols);
+    if (!config_size) {
+      setenvi("MINTTY_ROWS", term.rows0);
+      setenvi("MINTTY_COLS", term.cols0);
+      // provide environment to maximise window
+      if (win_is_fullscreen)
+        setenvi("MINTTY_MAXIMIZE", 2);
+      else if (IsZoomed(wnd))
+        setenvi("MINTTY_MAXIMIZE", 1);
     }
     // provide environment to select monitor
     if (moni > 0)
@@ -1077,13 +1167,14 @@ do_child_fork(int argc, char *argv[], int moni, bool launch)
 
 #if CYGWIN_VERSION_DLL_MAJOR >= 1005
     if (shortcut) {
-      //show_info(asform("Starting <%s>", cs__wcstoutf(shortcut)));
+      trace_dir(asform("Starting <%s>", cs__wcstoutf(shortcut)));
       shell_exec(shortcut);
       //show_info("Started");
       sleep(5);  // let starting settle, or it will fail; 1s normally enough
       exit(0);
     }
 
+    trace_dir(asform("Starting exe <%s %s>", argv[0], argv[1]));
     execv("/proc/self/exe", argv);
 #else
     // /proc/self/exe isn't available before Cygwin 1.5, so use argv[0] instead.
@@ -1098,16 +1189,16 @@ do_child_fork(int argc, char *argv[], int moni, bool launch)
 #endif
     exit(mexit);
   }
-  reset_fork_mode();
+  //reset_fork_mode();
 }
 
 /*
   Called from Alt+F2.
  */
 void
-child_fork(int argc, char *argv[], int moni)
+child_fork(int argc, char *argv[], int moni, bool config_size)
 {
-  do_child_fork(argc, argv, moni, false);
+  do_child_fork(argc, argv, moni, false, config_size);
 }
 
 /*
@@ -1150,7 +1241,7 @@ child_launch(int n, int argc, char * argv[], int moni)
           }
         }
         new_argv[argc] = 0;
-        do_child_fork(argc, new_argv, moni, true);
+        do_child_fork(argc, new_argv, moni, true, true);
         free(new_argv);
         break;
       }
@@ -1161,7 +1252,7 @@ child_launch(int n, int argc, char * argv[], int moni)
         // check for multi-line separation
         if (*cmdp == '\\' && cmdp[1] == '\n') {
           cmdp += 2;
-          while (isspace(*cmdp))
+          while (iswspace(*cmdp))
             cmdp++;
         }
       }
