@@ -58,11 +58,17 @@ typedef enum {
   SEL_COLOUR_I         = 279,
   SEL_TEXT_COLOUR_I    = 280,
 
-  // configured Bold colour
+  // configured attribute substitution colours
   BOLD_COLOUR_I = 281,
+  BLINK_COLOUR_I = 282,
+
+  // Tektronix colours
+  TEK_FG_COLOUR_I      = 283,
+  TEK_BG_COLOUR_I      = 284,
+  TEK_CURSOR_COLOUR_I  = 285,
 
   // Number of colours
-  COLOUR_NUM = 282,
+  COLOUR_NUM = 286,
 
   // True Colour indicator
   // assert (TRUE_COLOUR % 4) == 0 so that checking x >= TRUE_COLOUR
@@ -107,39 +113,45 @@ enum {
   ATTR_INVALID    = 0x0003FFFFu,
   ATTR_BOLD       = 0x00040000u,
   ATTR_DIM        = 0x00080000u,
-  ATTR_REVERSE    = 0x00100000u,
-  ATTR_UNDER      = 0x00200000u,
-  ATTR_BLINK      = 0x00400000u,
+  ATTR_REVERSE    = 0x00400000u,
+  ATTR_UNDER      = 0x02000000u,
+  ATTR_BLINK      = 0x00100000u,
 
   ATTR_ITALIC     = 0x00800000u,
   ATTR_INVISIBLE  = 0x01000000u,
-  ATTR_BLINK2     = 0x02000000u,
+  ATTR_BLINK2     = 0x00200000u,
   ATTR_STRIKEOUT  = 0x04000000u,
   ATTR_DOUBLYUND  = 0x08000000u,
   ATTR_OVERL      = 0x10000000u,
   ATTR_BROKENUND  = 0x0000000800000000u,
   ATTR_ULCOLOUR   = 0x0020000000000000u,
 
+  ATTR_CURLYUND   = ATTR_UNDER | ATTR_DOUBLYUND,
+  UNDER_MASK      = ATTR_UNDER | ATTR_DOUBLYUND | ATTR_BROKENUND,
+
+  ATTR_SHADOW     = 0x0000100000000000u,
+  ATTR_OVERSTRIKE = 0x0000200000000000u,
+  ATTR_SUBSCR     = 0x0000400000000000u,
+  ATTR_SUPERSCR   = 0x0000800000000000u,
+
   ATTR_PROTECTED  = 0x20000000u,
-  ATTR_WIDE       = 0x40000000u,
-  ATTR_NARROW     = 0x80000000u,
-  ATTR_EXPAND     = 0x0000000100000000u,
   ATTR_FRAMED     = 0x0010000000000000u,
 
-  TATTR_EMOJI     = 0x1000000000000000u,
-
-  GRAPH_MASK      = 0x0000FF0000000000u,
+  GRAPH_MASK      = 0x00000F0000000000u,
   ATTR_GRAPH_SHIFT = 40,
 
   FONTFAM_MASK    = 0x000F000000000000u,
   ATTR_FONTFAM_SHIFT = 48,
 
-  ATTR_CURLYUND   = ATTR_UNDER | ATTR_DOUBLYUND,
-  UNDER_MASK      = ATTR_UNDER | ATTR_DOUBLYUND | ATTR_BROKENUND,
+  TATTR_WIDE       = 0x40000000u,
+  TATTR_NARROW     = 0x80000000u,
+  TATTR_EXPAND     = 0x0000000100000000u,
+  TATTR_EMOJI     = 0x1000000000000000u,
 
   TATTR_COMBINING = 0x0000000200000000u, /* combining characters */
   TATTR_COMBDOUBL = 0x0000000400000000u, /* combining double characters */
   TATTR_ZOOMFULL  = 0x0000001000000000u, /* to be zoomed to full cell size */
+  TATTR_SINGLE    = 0x0040000000000000u, /* zoom down to single cell width */
 
   TATTR_RIGHTCURS = 0x0000002000000000u, /* cursor-on-RHS */
   TATTR_PASCURS   = 0x0000004000000000u, /* passive cursor (box) */
@@ -157,7 +169,6 @@ enum {
   DATTR_MASK      = TATTR_RIGHTCURS | TATTR_PASCURS | TATTR_ACTCURS
                     | DATTR_STARTRUN
   // unassigned bits:
-  //                0x0040000000000000u
   //                0x0080000000000000u
 };
 
@@ -169,17 +180,28 @@ enum {
   LATTR_TOP       = 0x0002u, /* DEC double-height line (DECDHL), top half */
   LATTR_BOT       = 0x0003u, /* DEC double-height line (DECDHL), bottom half */
   LATTR_MODE      = 0x0003u, /* mask for double-width/height attributes */
-  LATTR_WRAPPED   = 0x0010u, /* this line wraps to next */
-  LATTR_WRAPPED2  = 0x0020u, /* with WRAPPED: CJK wide character
+  LATTR_WRAPPED   = 0x4000u, /* this line wraps to next */
+  LATTR_WRAPPED2  = 0x8000u, /* with WRAPPED: CJK wide character
                                   * wrapped to next line, so last
                                   * single-width cell is empty */
-  LATTR_CLEARPAD  = 0x0040u, /* flag to clear padding from overhang */
-  LATTR_MARKED    = 0x0100u, /* scroll marker */
-  LATTR_UNMARKED  = 0x0200u, /* secondary scroll marker */
-  LATTR_NOBIDI    = 0x4000u, /* disable bidi on this line */
-  // overlay line display (italic right-to-left overhang handling):
-  LATTR_DISP1     = 0x1000u,
-  LATTR_DISP2     = 0x2000u,
+  LATTR_MARKED    = 0x0004u, /* scroll marker */
+  LATTR_UNMARKED  = 0x0008u, /* secondary scroll marker */
+  // bidi control
+  LATTR_WRAPCONTD = 0x2000u, /* continued from wrapped line */
+  LATTR_BIDIMASK  = 0x1FC0u, /* all bidi attributes */
+  LATTR_NOBIDI    = 0x0040u, /* disable bidi on this line / in paragraph */
+  LATTR_BOXMIRROR = 0x0080u, /* bidi box graphics mirroring */
+  LATTR_BIDISEL   = 0x0100u, /* direction pre-selected */
+  LATTR_BIDIRTL   = 0x0200u, /* direction (preset or fallback or explicit) */
+  // temporary bidi flags
+  LATTR_AUTOSEL   = 0x0400u, /* autodetection accomplished */
+  LATTR_AUTORTL   = 0x0800u, /* direction after autodetection */
+  // presentational bidi flag
+  LATTR_PRESRTL   = 0x1000u,
+  // enable automatic progress detection
+  LATTR_PROGRESS  = 0x0010u,
+  // unassigned bits:
+  //                0x0020u
 };
 
 enum {
@@ -192,9 +214,11 @@ typedef unsigned long long cattrflags;
 
 typedef struct {
   cattrflags attr;
-  uint truefg;
-  uint truebg;
+  colour truebg;
+  colour truefg;
   colour ulcolr;
+  int link;
+  int imgi;
 } cattr;
 
 extern const cattr CATTR_DEFAULT;
@@ -222,7 +246,7 @@ typedef struct {
   cattr attr;
 } termchar;
 
-/*const*/ termchar basic_erase_char;
+extern termchar basic_erase_char;
 
 typedef struct {
   ushort lattr;
@@ -246,8 +270,11 @@ extern int sblines(void);
 extern termline *fetch_line(int y);
 extern void release_line(termline *);
 
+
+/* Terminal state */
 typedef struct {
   int width;
+  ushort lattr;
   termchar *chars;
   int *forward, *backward;      /* the permutations of line positions */
 } bidi_cache_entry;
@@ -257,6 +284,7 @@ typedef enum {
   CSET_ASCII = 'B',   /* Normal ASCII charset */
   CSET_GBCHR = 'A',   /* UK variant */
   CSET_LINEDRW = '0', /* Line drawing charset */
+  CSET_VT52DRW = '2', /* VT52 "graphics" mode */
   CSET_TECH = '>',    /* DEC Technical */
   CSET_OEM = 'U',     /* OEM Codepage 437 */
   // definitions for DEC Supplemental support:
@@ -282,89 +310,145 @@ typedef enum {
   CSET_DEC_Greek_Supp		= '?' + 0x80,
   CSET_DEC_Hebrew_Supp		= '4' + 0x80,
   CSET_DEC_Turkish_Supp		= '0' + 0x80,
+  CSET_DEC_Cyrillic		= '&' + 0x80,
   CSET_NRCS_Greek		= '>' + 0x80,
   CSET_NRCS_Hebrew		= '=' + 0x80,
   CSET_NRCS_Turkish		= '2' + 0x80,
 } term_cset;
 
 typedef struct {
-  int y, x;
+  int y, x;      // cell coordinates of mouse event
+  int piy, pix;  // pixel coordinates of mouse event
   bool r;
 } pos;
 
+
 typedef enum {
   MBT_LEFT = 1, MBT_MIDDLE = 2, MBT_RIGHT = 3, MBT_4 = 4, MBT_5 = 5
 } mouse_button;
 
 
+/* Searching */
 typedef struct {
-  int x;
-  int y;
+  // Index of a virtual array of scrollback + screen.
+  // y = idx / term.cols
+  // x = idx % term.rows
+  // y starts from the top most line (y = 0, the first line of scrollback or screen).
+  int idx;
+  // The length of a match, maybe larger than term.results.xquery_length because of UCSWIDE.
   int len;
 } result;
 
 typedef struct {
+  // The current active result, for prev/next button.
+  result current;
+  // An idx can be matched against term.results.results iff idx in [range_begin, range_end).
+  int range_begin, range_end;
   result * results;
   wchar * query;
   xchar * xquery;
   int xquery_length;
+  // The capacity and length of results.
   int capacity;
-  int current;
   int length;
   int update_type;
 } termresults;
 
 
+/* Images */
 typedef struct {
-  void *fp;
+  void * fp;
   uint ref_counter;
   uint amount;
 } tempfile_t;
 
 typedef struct {
-  tempfile_t *tempfile;
+  tempfile_t * tempfile;
   size_t position;
 } temp_strage_t;
 
 typedef struct imglist {
-  unsigned char *pixels;
-  void *hdc;
-  void *hbmp;
-  temp_strage_t *strage;
-  int top;
-  int left;
-  int width;
-  int height;
-  int pixelwidth;
-  int pixelheight;
-  struct imglist *next;
+  // linked list
+  struct imglist * next;
+  struct imglist * prev;
+  // image ref for multiple use (currently unused)
+  char * id;
+  // sixel: rendering data
+  void * hdc;
+  void * hbmp;
+  // sixel: disk cache
+  temp_strage_t * strage;
+
+  // image data
+  unsigned char * pixels;
+  // image: data size; sixel: 0
+  int len;
+
+  // image ref for disposal management
+  int imgi;
+  // position within scrollback (top includes offset term.virtuallines)
+  int top, left;
+
+  // image area (cell units)
+  int width, height;
+  // sixel: image area size at time of output
+  // image: adjusted image size as requested, at time of output
+  int pixelwidth, pixelheight;
+  // image: terminal cell size at time of output
+  // sixel: actual graphic size, at time of output
+  int cwidth, cheight;
+  // image: cropping
+  int crop_x, crop_y, crop_width, crop_height;
+
+  // text attributes to be considered (blinking)
+  int attr;
 } imglist;
 
 typedef struct {
-  void *parser_state;
-  imglist *first;
-  imglist *last;
-  imglist *altfirst;
-  imglist *altlast;
+  void * parser_state;
+  imglist * first;
+  imglist * last;
+  imglist * altfirst;
+  imglist * altlast;
 } termimgs;
 
 
+enum {
+  MODO_1 = 1,
+  MODO_2 = 2,
+  MODO_ESC = 0x10,  /* Report ESC as CSI u */
+  MODO_MOD = 0x20,  /* Report plain modifiers */
+  MODO_REL = 0x40,  /* Report release */
+  MODO_REP = 0x80,  /* Report repeat */
+};
+
 typedef struct {
   short x, y;
-  cattr attr;
-  bool origin;
-  bool autowrap;  // switchable (xterm Wraparound Mode (DECAWM Auto Wrap))
   bool wrapnext;
-  bool rev_wrap;  // switchable (xterm Reverse-wraparound Mode)
+  cattr attr;
+  char width;  // handle explicit width attribute in termout.c
+  bool origin;
   short gl, gr;
   term_cset csets[4];
+  term_cset decsupp;
   term_cset cset_single;
   uchar oem_acs;
   bool utf;
-  bool decnrc_enabled;    /* DECNRCM sequence to enable NRC? */
+  ushort bidimode;
 } term_cursor;
 
+typedef struct {
+  int vol;
+  int last_vol;
+  unsigned long last_bell;
+} term_bell;
+
 struct term {
+  // these used to be in term_cursor, thus affected by cursor restore
+  bool decnrc_enabled;  /* DECNRCM: enable NRC */
+  bool autowrap;        /* DECAWM: Autowrap mode */
+  bool rev_wrap;        /* xterm: Reverse wraparound mode */
+
   bool on_alt_screen;     /* On alternate screen? */
   bool show_other_screen;
 
@@ -389,9 +473,12 @@ struct term {
   char * suspbuf;         /* suspend output during selection buffer */
   uint suspbuf_size, suspbuf_pos;
 
+  int suspend_update;
+
   bool rvideo;            /* global reverse video flag */
   bool cursor_on;         /* cursor enabled flag */
   bool deccolm_allowed;   /* DECCOLM sequence for 80/132 cols allowed? */
+  bool deccolm_noclear;   /* DECCOLM does not clear screen */
   bool reset_132;         /* Flag ESC c resets to 80 cols */
   bool cblinker;          /* When blinking is the cursor on ? */
   bool tblinker;          /* When the blinking text is on */
@@ -399,7 +486,10 @@ struct term {
   bool blink_is_real;     /* Actually blink blinking text */
   bool echoing;           /* Does terminal want local echo? */
   bool insert;            /* Insert mode */
-  int marg_top, marg_bot; /* scroll margins */
+  int marg_top, marg_bot; /* scrolling region margins */
+  int marg_left, marg_right; /* horizontal margins */
+  bool lrmargmode;           /* enable horizontal margins */
+  bool attr_rect;            /* rectangular attribute change extent */
   bool printing, only_printing;  /* Are we doing ANSI printing? */
   int  print_state;       /* state of print-end-sequence scan */
   char *printbuf;         /* buffered data for printer */
@@ -411,6 +501,10 @@ struct term {
   bool focus_reported;
   bool in_vbell;
 
+  term_bell bell, marginbell;
+  bool margin_bell;
+  bool ring_enabled;
+
   bool vt220_keys;
   bool shortcut_override;
   bool backspace_sends_bs;
@@ -420,10 +514,13 @@ struct term {
   unsigned int app_control;
   bool app_cursor_keys;
   bool app_keypad;
-  bool app_wheel;
+  bool auto_repeat;
+  int repeat_rate;
   bool bell_taskbar; // xterm: bellIsUrgent; switchable with CSI ? 1042 h/l
   bool bell_popup;   // xterm: popOnBell;    switchable with CSI ? 1043 h/l
-  bool wheel_reporting;
+  bool wheel_reporting_xterm; // xterm: alternateScroll
+  bool wheel_reporting;       // similar, but default true
+  bool app_wheel;             // format for wheel_reporting
   int  modify_other_keys;
   bool newline_mode;
   bool report_focus;
@@ -431,10 +528,12 @@ struct term {
   bool report_ambig_width;
   bool bracketed_paste;
   bool show_scrollbar;
+  bool app_scrollbar;
   bool wide_indic;
   bool wide_extra;
   bool disable_bidi;
   bool enable_bold_colour;
+  bool enable_blink_colour;
 
   bool sixel_display;        // true if sixel scrolling mode is off
   bool sixel_scrolls_right;  // on: sixel scrolling leaves cursor to right of graphic
@@ -443,12 +542,17 @@ struct term {
                              // off(default): sixel scrolling moves cursor to left of graphics
   bool private_color_registers;
   int  cursor_type;
+  int  cursor_size;
+  bool cursor_blinkmode;
   int  cursor_blinks;
+  int  cursor_blink_interval;
   bool cursor_invalid;
   bool hide_mouse;
 
   uchar esc_mod;  // Modifier character in escape sequences
 
+  uchar vt52_mode;
+
   uint csi_argc;
   uint csi_argv[32];
   uint csi_argv_defined[32];
@@ -460,6 +564,12 @@ struct term {
   int dcs_cmd;
 
   uchar *tabs;
+  bool newtab;
+
+  bool iso_guarded_area;  // global distinction of ISO Guarded Area
+                          // as protected (xterm-like simplification)
+
+  int detect_progress;
 
   enum {
     NORMAL, ESCAPE, CSI_ARGS,
@@ -472,7 +582,10 @@ struct term {
     DCS_INTERMEDIATE,
     DCS_PASSTHROUGH,
     DCS_IGNORE,
-    DCS_ESCAPE
+    DCS_ESCAPE,
+    VT52_Y, VT52_X,
+    VT52_FG, VT52_BG,
+    TEK_ESCAPE, TEK_ADDRESS0, TEK_ADDRESS, TEK_INCREMENTAL
   } state;
 
   // Mouse mode
@@ -490,7 +603,8 @@ struct term {
     ME_X10,        // CSI M followed by one byte each for event, X and Y
     ME_UTF8,       // Same as X10, but with UTF-8 encoded X and Y (ugly!)
     ME_URXVT_CSI,  // CSI event ; x ; y M
-    ME_XTERM_CSI   // CSI > event ; x ; y M/m
+    ME_XTERM_CSI,  // CSI > event ; x ; y M/m
+    ME_PIXEL_CSI   // CSI > event ; pix ; piy M/m
   } mouse_enc;
 
   enum {
@@ -509,6 +623,7 @@ struct term {
   bool selected, sel_rect;
   pos sel_start, sel_end, sel_anchor;
   bool hovering;
+  int hoverlink;
   pos hover_start, hover_end;
 
  /* Scroll steps during selection when cursor out of window. */
@@ -542,14 +657,16 @@ struct term {
 
 extern struct term term;
 
+extern void scroll_rect(int topline, int botline, int lines);
+
 extern void term_resize(int, int);
 extern void term_scroll(int, int);
 extern void term_reset(bool full);
 extern void term_clear_scrollback(void);
-extern void term_mouse_click(mouse_button, mod_keys, pos, int count);
+extern bool term_mouse_click(mouse_button, mod_keys, pos, int count);
 extern void term_mouse_release(mouse_button, mod_keys, pos);
 extern void term_mouse_move(mod_keys, pos);
-extern void term_mouse_wheel(int delta, int lines_per_notch, mod_keys, pos);
+extern void term_mouse_wheel(bool horizontal, int delta, int lines_per_notch, mod_keys, pos);
 extern void term_select_all(void);
 extern void term_paint(void);
 extern void term_invalidate(int left, int top, int right, int bottom);
@@ -567,7 +684,6 @@ extern void term_write(const char *, uint len);
 extern void term_flush(void);
 extern void term_set_focus(bool has_focus, bool may_report);
 extern int  term_cursor_type(void);
-extern bool term_cursor_blinks(void);
 extern void term_hide_cursor(void);
 
 extern void term_set_search(wchar * needle);
@@ -576,5 +692,8 @@ extern void term_schedule_search_update(void);
 extern void term_update_search(void);
 extern void term_clear_results(void);
 extern void term_clear_search(void);
+extern void term_search_expand(int idx);
+extern result term_search_prev(void);
+extern result term_search_next(void);
 
 #endif
