@@ -24,13 +24,14 @@ extern void win_update(bool update_sel_tip);
 extern void win_schedule_update(void);
 extern void do_update(void);
 
-extern void win_text(int x, int y, wchar *text, int len, cattr attr, cattr *textattr, ushort lattr, bool has_rtl);
+extern void win_text(int x, int y, wchar *text, int len, cattr attr, cattr *textattr, ushort lattr, bool has_rtl, bool clearpad, uchar phase);
 
 extern void win_update_mouse(void);
 extern void win_capture_mouse(void);
 extern void win_get_locator_info(int *x, int *y, int *buttons, bool by_pixels);
 
 extern void win_bell(config *);
+extern void win_margin_bell(config *);
 
 extern void win_set_title(char *);
 extern void win_save_title(void);
@@ -42,7 +43,7 @@ extern void win_copy_text(const char *s);
 extern colour win_get_colour(colour_i);
 extern void win_set_colour(colour_i, colour);
 extern void win_reset_colours(void);
-extern colour win_get_sys_colour(bool fg);
+extern colour win_get_sys_colour(int colid);
 extern uint colour_dist(colour a, colour b);
 extern colour truecolour(cattr *, colour bg);
 
@@ -60,17 +61,21 @@ extern void win_get_scrpos(int *xp, int *yp, bool with_borders);
 extern void win_get_pixels(int *height_p, int *width_p, bool with_borders);
 extern void win_get_screen_chars(int *rows_p, int *cols_p);
 extern void win_popup_menu(mod_keys mods);
+extern bool win_title_menu(bool leftbut);
 
 extern void win_zoom_font(int, bool sync_size_with_font);
 extern void win_set_font_size(int, bool sync_size_with_font);
 extern uint win_get_font_size(void);
 
-extern void win_check_glyphs(wchar *wcs, uint num);
+extern void win_check_glyphs(wchar *wcs, uint num, cattrflags attr);
+extern int win_char_width(xchar, cattrflags attr);
+extern wchar win_combine_chars(wchar bc, wchar cc, cattrflags attr);
 
 extern void win_open(wstring path, bool adjust_dir);
 extern void win_copy(const wchar *data, cattr *cattrs, int len);
 extern void win_copy_as(const wchar *data, cattr *cattrs, int len, char what);
 extern void win_paste(void);
+extern void win_paste_path(void);
 
 extern void win_set_timer(void_fn cb, uint ticks);
 
@@ -88,9 +93,6 @@ extern void win_clear_images(void);
 extern int get_tick_count(void);
 extern int cursor_blink_ticks(void);
 
-extern int win_char_width(xchar);
-extern wchar win_combine_chars(wchar bc, wchar cc);
-
 extern wchar win_linedraw_char(int i);
 
 typedef enum {
